#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------
# Powershell Script for Windows Server 2012 Hyper-V, Provisioning Services XenDesktop and Streamed VM Setup Wizard alternative
# This script is designed to assist in creating Hyper-V 3 virtual machines for PVS to be used in combination with XenDesktop
# WS2012_Hyper-V_PVS_XenDesktop_StreamedVM_Setup_Wizard.ps1 
# Variables VMNAME, DESTINATIONPATH, CACHEVHDFORMAT, NAMINGPREFIX, and NUMBEROFVMS must be configured prior to use
#
# This script can be executed from a standalone Hyper-V host to create a predefined number of VMs for use with PVS and XenDesktop
# Error checking has not yet been built into this script, so please check and double check all variables
# Created by Dane Young, Entisys Solutions Copyright 2012
# Build 2012.9.8 Revision 1
# For distribution revisions go to: http://itvce.com
#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------

$templatename = "Template" 			# Define the VM name of desired template to be used for cloning additional VMs
$destinationpath = "D:\Virtual Machines"	# Destination path on Hyper-V host where VMs and VHD, VHDX files should be stored
$namingprefix = "ITVCEHV3-1"			# Naming convention for creating virtual machines. This script will add 3 incremental digits
$numberofvms = 20				# Define the number of virtual machines to be created from template

#-----------------------------------------------------------------------------------

write-host (get-date -uformat %I:%M:%S) "- Processing initialized. Creating" $numberofvms "virtual machines from" $templatename"." -ForegroundColor Green

# Capture cache disk information from template VM
$path = (Get-vmharddiskdrive $templatename).path
$cachevhdformation = [System.IO.Path]::GetExtension($path)
$ctype = (Get-vmharddiskdrive $templatename).controllertype
$cnum = (Get-vmharddiskdrive $templatename).controllernumber
$cloc = (Get-vmharddiskdrive $templatename).controllerlocation
$temp = [environment]::GetEnvironmentVariable("temp","machine") + "\"

# In order to clone additional VMs, this cache disk needs to be detached, then reattached after all VMs have been created
remove-vmharddiskdrive $templatename -ControllerType $ctype -ControllerNumber $cnum -ControllerLocation $cloc
# Export the template VM to C:\Windows\Temp. Deleting the directory if it already exists
If(Test-Path -Path ($temp + $templatename)){Remove-item ($temp + $templatename) -Recurse}
export-vm $templatename $temp

# Capture the VM configuration file from the export directory within system temp directory
foreach ($xml in Get-ChildItem ($temp + $templatename + "\Virtual Machines") -filter "*.xml"){$importpath = $temp + $templatename + "\Virtual Machines\" + $xml}

# Create a log file for storing all VM names, MAC Addresses, Site, and Collection information for importing into PVS
If((Test-Path -Path ($temp + $namingprefix + ".log"))){Remove-item ($temp + $namingprefix + ".log")}

# Check to see if VMs already exist with the same naming convention. If VMs already exist, start incrementing from the last known VM number
get-vm | sort-object Name | where {$_.Name -like $namingprefix + "*"} | Select-Object -last 1 | foreach-object {$lastvm = $_.Name.Substring($_.Name.Length - 3,3)}
if ( $lastvm -ne $null ){ $i = [int]$lastvm} else { $i = 0}
$j = $i + $numberofvms

    # Loop through the following subprocedure to create the desired number of VMs using i and j as the starting and ending numbers
    for ($i += 1; $i -le $j; $i++) {

    # Create a name for the VM to be imported
    $newvm = $namingprefix + "{0:d3}" -f $i

    # Import a VM from system temp directory using the new VM name defined above
    import-vm $importpath -GenerateNewID -Copy -SnapshotFilePath ($destinationpath + "\" + $newvm) -SmartPagingFilePath ($destinationpath + "\" + $newvm) -VirtualMachinePath ($destinationpath + "\" + $newvm) | Rename-VM -NewName $newvm
    # Check to see if the Virtual Hard Disks path exists within the new VM's destination path
    If((Test-Path -Path ($destinationpath + "\" + $newvm + "\Virtual Hard Disks\")) -eq $false){$mkdir = mkdir ($destinationpath + "\" + $newvm + "\Virtual Hard Disks\")}

    # Copy the cache disk from the original template to the newly created VM's destination path
    copy $path ($destinationpath + "\" + $newvm + "\Virtual Hard Disks\")
    # Attach the newly copied VHD or VHDX cache disk to the newly create VM
    foreach ($vhd in Get-ChildItem ($destinationpath + "\" + $newvm + "\Virtual Hard Disks\") -filter ("*" + $cachevhdformat)){$vhdname = $vhd.name}
    Add-VMHardDiskDrive -VM (get-vm $newvm) -Controllertype $ctype -Controllernumber $cnum -Controllerlocation $cloc -Path ($destinationpath + "\" + $newvm + "\Virtual Hard Disks\" + $vhdname)

    # Start the new virtual machine. This step is required as the MAC addresses do not get assigned from the pool until the VM is first powered on
    start-vm $newvm
    # Capture the MAC address from the legacy network adapter
    $macaddress = (Get-VMNetworkAdapter -vmname $newvm -IsLegacy $true).MacAddress
    # Force stop the new virtual machine
    stop-vm $newvm -Force

    # Dump the VM name and MAC address to the log file and give feedback to the Powershell runspace
    Add-Content ($temp + $namingprefix + ".log") "$newvm,$macaddress,Site,Collection,"
    write-host (get-date -uformat %I:%M:%S) "- Successfully created" $newvm" with the following Mac Address "$macaddress -ForegroundColor Green 
    }

# After the loop of virtual machines has completed, attach the original cache disk back to the template virtual machine
Add-VMHardDiskDrive -VM (get-vm $templatename) -Controllertype $ctype -Controllernumber $cnum -Controllerlocation $cloc -Path $path

# Give feedback upon completion and open up the log file to be imported into PVS
write-host (get-date -uformat %I:%M:%S) "- Processing completed." -ForegroundColor Green 
notepad.exe ($temp + $namingprefix + ".log")

#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------
# Created by Dane Young, Entisys Solutions Copyright 2012
# Build 2012.9.8 Revision 1
# For distribution revisions go to: http://itvce.com
#-----------------------------------------------------------------------------------
#-----------------------------------------------------------------------------------